import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.softquadrat.datasqill.entity.validate.Action;
import de.softquadrat.datasqill.entity.validate.ObjectDs;
import de.softquadrat.datasqill.modules.datasqillDBConnection;
import de.softquadrat.datasqill.modules.datasqillTransformationRequest;

/**
 * This class is used as base class for the DsModDemo
 * module. It provides several helper methods that are inherited
 * for usage.
 */
public class BaseModule {
	/** The logger used by this module. */
	protected static final Logger LOGGER = Logger.getLogger("de.softquadrat.datasqill.modules");

	/**
	 * Gets the action from the incoming request.
	 * 
	 * @param request the incoming request.
	 * @return the action.
	 * @throws DsModException in case of any errors.
	 */
	protected Action getAction(datasqillTransformationRequest request) throws DsModException {
		Action action = request.getAction();
		if (action == null)
			throw new DsModException(DsModErrorCodes.ERR_CODE_MISSING_ACTION);
		LOGGER.fine("action: " + action);
		return action;
	}

	/**
	 * Gets the action string i.e. the sql query.
	 * 
	 * @param action the action containing the action string.
	 * @return the action string i.e. the sql query.
	 * @throws DsModException in case of any errors.
	 */
	protected String getActionString(Action action) throws DsModException {
		String actionString = action.getAction();
		String result;
		if (actionString == null || (result = actionString.trim()).length() == 0)
			throw new DsModException(DsModErrorCodes.ERR_CODE_MISSING_ACTION_STRING);
		LOGGER.fine("actionString: " + actionString);
		return result;
	}

	/**
	 * Gets the source tables for the transformation.
	 * 
	 * @param action the action containing the source tables.
	 * @return the source tables.
	 * @throws DsModException in case of any errors.
	 */
	protected List<ObjectDs> getSources(Action action) throws DsModException {
		List<ObjectDs> sourceTables = action.getSources();
		for (ObjectDs source : sourceTables) {
			LOGGER.fine("source: " + source.getObjectOwner() + "/" + source.getObjectName());
			if (source.getSqtsObjectTypeId() != 1L)
				throw new DsModException(DsModErrorCodes.ERR_CODE_WRONG_SOURCE, ", got type " + source.getSqtsObjectTypeId());			
		}
		return sourceTables;
	}

	/**
	 * Gets the database id from the source tables.
	 * The database id must be the same for all source tables. An exception is thrown
	 * if this is not the case.
	 * 
	 * @param sourceTables the source tables.
	 * @return the database id
	 * @throws DsModException in case of any errors.
	 */
	protected Long getDatabaseId(List<ObjectDs> sourceTables) throws DsModException {
		if (sourceTables.size() == 0)
			throw new DsModException(DsModErrorCodes.ERR_CODE_MISSING_SOURCE);
		Long serverId = sourceTables.get(0).getServerId();
		for (ObjectDs source : sourceTables) {
			if (serverId != source.getServerId())
				throw new DsModException(DsModErrorCodes.ERR_CODE_DIFFERENT_DBS);
		}
		return serverId;
	}
	
	/**
	 * Gets the csv target file.
	 * @param action the action containing the csv target file.
	 * @return the csv target file.
	 * @throws DsModException in case of any errors.
	 */
	protected ObjectDs getTarget(Action action) throws DsModException {
		List<ObjectDs> targets = action.getTargets();
		if (targets.size() == 0)
			throw new DsModException(DsModErrorCodes.ERR_CODE_MISSING_TARGET);
		if (targets.size() > 1)
			throw new DsModException(DsModErrorCodes.ERR_CODE_TWO_MANY_TARGETS);
		ObjectDs target = targets.get(0);
		if (target.getSqtsObjectTypeId() != 7L)
			throw new DsModException(DsModErrorCodes.ERR_CODE_WRONG_TARGET, ", got type " + target.getSqtsObjectTypeId());
		LOGGER.fine("target: " + target.getServerId() + " " + target.getObjectOwner() + "." + target.getObjectName());
		return target;
	}

	/**
	 * Gets the command for the module invocation like "run" or "validate".
	 * 
	 * @param request the incoming request.
	 * @return the command.
	 * @throws DsModException in case of any errors.
	 */
	protected String getCommand(datasqillTransformationRequest request) throws DsModException {
		String command = request.getCommand();
		if (command == null)
			throw new DsModException(DsModErrorCodes.ERR_CODE_MISSING_COMMAND);
		LOGGER.fine("command: " + command);
		return command;
	}

	/**
	 * Opens a database connection for a database id.
	 * 
	 * @param databaseId the database id.
	 * @return the database connection.
	 * @throws DsModException in case of any errors.
	 */
	protected datasqillDBConnection openConnection(Long databaseId) throws DsModException {
		try {
			datasqillDBConnection connection = datasqillDBConnection.openConnection(databaseId);
			if (connection == null)
				throw new DsModException(DsModErrorCodes.ERR_CODE_CONNECTION_PROBLEM);
			return connection;
		} catch (Exception e) {
			LOGGER.log(Level.SEVERE, e.getMessage(), e);
			throw new DsModException(DsModErrorCodes.ERR_CODE_CONNECTION_PROBLEM);
		}
	}
}
