import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;

import de.softquadrat.datasqill.entity.validate.Action;
import de.softquadrat.datasqill.entity.validate.ObjectDs;
import de.softquadrat.datasqill.modules.datasqillDBConnection;
import de.softquadrat.datasqill.modules.datasqillDBValidate;
import de.softquadrat.datasqill.modules.datasqillProgram;
import de.softquadrat.datasqill.modules.datasqillProgramStart;
import de.softquadrat.datasqill.modules.datasqillStatistic;
import de.softquadrat.datasqill.modules.datasqillTransformationAnswer;
import de.softquadrat.datasqill.modules.datasqillTransformationRequest;

/**
 * Instances of this module class are used to write a csv file based on an SQL
 * query.
 */
public class DsModDemo extends BaseModule implements datasqillProgram {
	/**
	 * Executes the run and transforms the data.
	 *
	 * @param dbCon     the database connection of the source tables.
	 * @param target    the csv target file.
	 * @param sqlQuery  the sql query used to retrieve the data.
	 * @param delimiter the csv delimiter.
	 * @param append    if true, then lines will be written to the end of the file
	 *                  rather than the beginning
	 * @return the number of transformed data rows.
	 * @throws DsModException in case of any errors.
	 */
	private int doRun(datasqillDBConnection dbCon, ObjectDs target, String sqlQuery, String delimiter, boolean append)
			throws DsModException {
		// get target
		String directory = target.getObjectOwner();
		String filename = target.getObjectName();
		File file = new File(directory, filename);
		// 
		try (FileWriter fw = new FileWriter(file, append);
				BufferedWriter bw = new BufferedWriter(fw);
				PreparedStatement preparedStatement = dbCon.prepareStatement(sqlQuery);
				ResultSet resultSet = preparedStatement.executeQuery()) {
			// write csv header line
			ResultSetMetaData metaData = resultSet.getMetaData();
			int columnCount = metaData.getColumnCount();
			for (int i = 1; i <= columnCount; i++) {
				bw.write(metaData.getColumnLabel(i));
				if (i < columnCount)
					bw.write(delimiter);
			}
			bw.write("\n");
			// transfer data row by row to csv target file
			int count = 0;
			while (resultSet.next()) {
				for (int i = 1; i <= columnCount; i++) {
					bw.write(resultSet.getString(i));
					if (i < columnCount)
						bw.write(delimiter);
				}
				bw.write("\n");
				count++;
			}
			// and return number of rows processed
			return count;
		} catch (Exception e) {
			LOGGER.log(Level.SEVERE, e.getMessage(), e);
			throw new DsModException(DsModErrorCodes.ERR_CODE_TRANSFORM_PROBLEM);
		}
	}

	/**
	 * Validates the query and the graph model.
	 *
	 * @param response      the response of the module invocation.
	 * @param dbConSourceDb the source database connection.
	 * @param sqlQuery      the sql query.
	 * @return true if validation was successful.
	 */
	private boolean doValidate(datasqillTransformationAnswer response, datasqillDBConnection dbConSourceDb,
			String sqlQuery) {
		boolean result = datasqillDBValidate.validateSQLStatement(dbConSourceDb, response, sqlQuery);
		if (result)
			datasqillDBValidate.setStatementDependency(dbConSourceDb, response,
					"SELECT * FROM (" + sqlQuery + ") query WHERE 1=0");
		LOGGER.info("Validation result: " + result);
		return result;
	}

	/**
	 * Executes the modules functionality:
	 * <ul>
	 * <li>transforms the data according to the sql query given by the datasqill
	 * developer or</li>
	 * <li>validates the sql query and verifies it against the model.</li>
	 * </ul>
	 * <p>
	 * The execution is done in following steps
	 * <ol>
	 * <li>start time measuring</li>
	 * <li>read parameters</li>
	 * <li>open database connection</li>
	 * <li>execute data transformation or</li>
	 * <li>validate sql query and model</li>
	 * <li>add result data to response</li>
	 * <li>return response</li>
	 * <p>
	 * In case of any error the error code and error message are added to the
	 * repsonse.
	 */
	@Override
	public datasqillTransformationAnswer execute(datasqillTransformationRequest request) {
		System.out.println("StdOut Message");
		System.err.println("StdErr Message");
		LOGGER.info("Starting module");
		System.out.println("Starting module");
		// start time measuring and create response
		long startTime = System.nanoTime();
		datasqillTransformationAnswer response = new datasqillTransformationAnswer(request.getRequestId(), 0,
				"DsModFilterCsvFile", null, 0, null, -1, null, null);
		try {
			// read parameters
			Action action = getAction(request);
			String sqlQuery = getActionString(action);
			String cmd = getCommand(request);
			ObjectDs target = getTarget(action);
			List<ObjectDs> sources = getSources(action);
			Long targetDatabaseID = getDatabaseId(sources);
			String delimiter = action.getAttributesString("Delimiter", ",");
			boolean append = action.getAttributesChar("Append", 'N') == 'Y';
			// open database connection
			datasqillDBConnection dbCon = openConnection(targetDatabaseID);
			// run command
			switch (cmd) {
			case "Run":
				// execute data transformation
				int count = doRun(dbCon, target, sqlQuery, delimiter, append);
				response.setRowsProcessed(count);
				break;
			case "Validate":
				// validate sql query and model
				doValidate(response, dbCon, sqlQuery);
				break;
			default:
				throw new DsModException(DsModErrorCodes.ERR_CODE_MISSING_COMMAND, ", got command " + cmd);
			}
			// add result data to response
			long hostId = action.getExecutingHostId();
			long moduleDuration = System.nanoTime() - startTime;
			List<datasqillStatistic> statistics = new ArrayList<>();
			statistics.add(new datasqillStatistic(hostId, "module_duration", moduleDuration));
			response.setStatistics(statistics);
		} catch (DsModException e) {
			response.setErrorCode(e.getErrorCode());
			response.setErrorMessage(e.getErrorMessage());
		}
		LOGGER.info("Ending module");
		System.out.println("Ending module");
		// return response
		return response;
	}

	/**
	 * The main method, invoked when the module is called by datasqill server.
	 *
	 * @param args invocation arguments passed to the module.
	 */
	public static void main(String args[]) {
		datasqillProgramStart.init(args, new DsModDemo());
	}
}
