import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Statement;

import org.junit.Before;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.databind.JsonMappingException;

import de.softquadrat.datasqill.modules.ConnectionInfo;
import de.softquadrat.datasqill.modules.ConnectionReader;
import de.softquadrat.datasqill.modules.datasqillDBConnection;
import de.softquadrat.datasqill.serverinterface.ServerRequest;

/**
 * This class is used as base class for module testing classes.
 *
 */
public class BaseTestModule {
	/**
	 * Instances of this class are used to emulated the connection retrieval via
	 * "getkey".
	 *
	 */
	public static class MyConnectionReader extends ConnectionReader {
		@Override
		public ConnectionInfo readConnectionInfo(String key) {
			return new ConnectionInfo("jdbc:h2:mem:test", "SQTS_USER", "softquadrat", new String[0]);
		}
	}

	/**
	 * Initializes the in memory H2 database used for testing.
	 * 
	 * @throws Exception in case of any errors.
	 */
	@Before
	public void init() throws Exception {
		ConnectionReader.setInstance(new MyConnectionReader());
		datasqillDBConnection dsConnection = datasqillDBConnection.openConnection(1);
		Statement statement = dsConnection.createStatement();
		statement.execute("DROP TABLE IF EXISTS public.person");
		statement.execute("CREATE TABLE public.person (\n" + "    id integer NOT NULL\n" + "  , fname varchar\n"
				+ "  , lname character varying\n" + ")");
		statement.execute("INSERT INTO public.person (id,fname,lname) VALUES \n" + "(2,'Abraham','Schmidt')\n"
				+ ",(3,'Achaz','Schneider')\n" + ",(4,'Achim','Fischer')\n" + ";");
		dsConnection.commit();
		statement.close();
	}

	/**
	 * Loads a server request from a Json file.
	 * 
	 * @param jsonFile the Json file.
	 * @return the server request.
	 * @throws JsonParseException   thrown on parsing problems, used when
	 *                              non-well-formed content (content that does not
	 *                              conform to JSON syntax as per specification) is
	 *                              encountered.
	 * @throws JsonMappingException thrown to signal fatal problems with mapping of
	 *                              content.
	 * @throws IOException          in case of problems reading the Json file.
	 */
	public ServerRequest loadServerRequest(String jsonFile)
			throws JsonParseException, JsonMappingException, IOException {
		try (FileReader fr = new FileReader(jsonFile); BufferedReader br = new BufferedReader(fr)) {
			String line;
			StringBuffer sb = new StringBuffer();
			while ((line = br.readLine()) != null) {
				sb.append(line);
				sb.append("\n");
			}
			ServerRequest json = ServerRequest.fromJson(sb.toString());
			return json;
		}
	}
}
