import java.io.File;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;

import org.junit.Assert;
import org.junit.Test;

import de.softquadrat.datasqill.modules.Module;
import de.softquadrat.datasqill.modules.WrapperModule;
import de.softquadrat.datasqill.serverinterface.ServerRequest;
import de.softquadrat.datasqill.serverinterface.ServerResponse;
import de.softquadrat.datasqill.serverinterface.ServerResponse.Error;

/**
 * This test class for module DsModDemo contains some test methods to
 * verify validate and run executions.
 * 
 * The class uses a in memory H2 database with some test table, that is
 * set up before tests are started.
 *
 */
public class TestDsModDemo extends BaseTestModule {
	/** The error message for failing validation. */
	private static final String MSG_TEXT = "Column \"XXXX\" not found";
	/** The expected file contents after run. */
	private static final String FILE_CONTENT = "ID,FNAME,LNAME\n" + 
			"2,Abraham,Schmidt\n" + 
			"3,Achaz,Schneider\n" + 
			"4,Achim,Fischer\n";

	/**
	 * Tests a successful validation.
	 * 
	 * @throws Exception in case of any errors.
	 */
	@Test
	public void testValidateSuccess() throws Exception {
		ServerRequest request = loadServerRequest("src/test/resources/validateSuccess.json");
		DsModDemo module = new DsModDemo();
		Module wrapperModule = new WrapperModule(module);
		ServerResponse response = wrapperModule.execute(request);
		String json = response.toPrettyJson();
		System.out.println(json);
		Assert.assertNull("Error must be null", response.getError());
	}

	/**
	 * Tests a failing validation.
	 * 
	 * @throws Exception in case of any errors.
	 */
	@Test
	public void testValidateFailed() throws Exception {
		ServerRequest request = loadServerRequest("src/test/resources/validateFailed.json");
		DsModDemo module = new DsModDemo();
		Module wrapperModule = new WrapperModule(module);
		ServerResponse response = wrapperModule.execute(request);
		String json = response.toPrettyJson();
		System.out.println(json);
		Error error = response.getError();
		Assert.assertEquals("Error code expected", (long) error.getErrorCode(), 42122L);
		Assert.assertEquals("Error source expected", error.getErrorSource(), "validateSQLStatement");
		Assert.assertTrue("Error message expected", error.getErrorMessage().contains(MSG_TEXT));
		
	}

	/**
	 * Tests a successful run execution.
	 * 
	 * @throws Exception in case of any errors.
	 */
	@Test
	public void testRun() throws Exception {
		File outFile = new File("output");
		outFile.delete();
		ServerRequest request = loadServerRequest("src/test/resources/run.json");
		DsModDemo module = new DsModDemo();
		Module wrapperModule = new WrapperModule(module);
		ServerResponse response = wrapperModule.execute(request);
		System.out.println(response.toPrettyJson());
		Assert.assertTrue("Output file missing", outFile.exists());
		byte[] bytes = Files.readAllBytes(outFile.toPath());
		String contents = new String(bytes, StandardCharsets.UTF_8);
		Assert.assertEquals("Wrong file content found", contents, FILE_CONTENT);
	}
}
